<?php
namespace app\tuchuang\controller;

use app\common\controller\Front;

class Index extends Front
{
    //远程图片地址
    private $imageUrl   = '';
    //图片内容
    private $imgContent = '';
    //图片内型contentType
    private $mimeType   = '';
    //图片临时文件名
    private $fileName   = 'test';

	public function _initialize()
    {
        $this->imageUrl = DcEmpty(config('tuchuang.link_default'), 'https://cdn.daicuo.cc/images/gzh/dc_01.jpg');
        
		parent::_initialize();
	}

	public function index()
    {
        if( $this->request->isAjax() ){
        
            config('common.app_domain',true);
            
            if(!$url = input('request.url/s')){
                return json(['code'=>0,'msg'=>lang('tc_error_url_empty'),'data'=>'']);
            }
            
            $url = $this->urlEncode($url);
            
            return json(['code'=>1,'msg'=>'ok','data'=>DcUrl('tuchuang/index/read',['url'=>$url])]);
        }
		return $this->fetch();
	}
    
    public function read()
    {
        //解密图片地址
        $imgUrl = $this->urlDecode(input('request.url/s'));

        //验证协议
        $args = parse_url($imgUrl);
        if(!in_array($args['scheme'], array('http','https','ftp'))){
            $this->redirect($imgUrl, 302);
        }
        
        //验证访问域名
        if( config('tuchuang.link_domain') ){
            $referer = parse_url(input('server.HTTP_REFERER')); 
            $domains = explode(',', config('tuchuang.link_domain'));
            if( !in_array($referer['host'], $domains) ){
                $this->redirect($imgUrl, 302);
            }
        }
        
        //验证加速域名
        if( config('tuchuang.link_referer') ){
            $referers = explode(',', config('tuchuang.link_referer'));
            if( !in_array($args['host'], $referers) ){
                $this->redirect($imgUrl, 302);
            }
        }
        
        //图片缓存状态
        if(config('tuchuang.cache_time') > 0){
            return $this->imgCache($imgUrl);
        }elseif(config('tuchuang.cache_time') < 0){
            return $this->imgFile($imgUrl);
        }else{
            $this->curl($imgUrl);
            return $this->echoImg();
        }
	}
    
    //图片保存到缓存
    private function imgCache($imgUrl='')
    {
        //图片缓存相关
        $this->fileName = md5($imgUrl);
        //是否启缓缓存
        $cacheTime = config('tuchuang.cache_time');
        //优先缓存读取(图片内型与图片内容)
        $imgCache = DcCache($this->fileName);
        if($imgCache['mimeType'] && $imgCache['imgContent']){
            $this->mimeType   = $imgCache['mimeType'];
            $this->imgContent = $imgCache['imgContent'];
        }else{
            //CURL请求远程图片
            $this->curl($imgUrl);
            //将图片缓存到本地
            if($this->imgContent && $this->mimeType){
                DcCache($this->fileName, ['mimeType'=>$this->mimeType,'imgContent'=>$this->imgContent], intval($cacheTime));
            }
        }
        //输出图片
        return $this->echoImg();
	}
    
    //图片保存到本地
    private function imgFile($imgUrl='')
    {
        //md5文件名
        $this->fileName = md5($imgUrl);
        //保存文件名
        $saveName = ltrim(config('common.upload_path'), '/').'/'.substr($this->fileName,0,2).'/'.$this->fileName;
        //实例化文件操作类
        $file = new \files\File();
        //判断文件是否存在
        if($file->f_has($saveName)){
            $this->mimeType   = imgMimeType($saveName);
            $this->imgContent = $file->read($saveName);
        }else{
            //CURL请求远程图片
            $this->curl($imgUrl);
            //下载远程图片到本地
            if($this->imgContent && $this->mimeType){
                $file->write($saveName, $this->imgContent);
            }
        }
        //输出图片
        return $this->echoImg();
    }
    
    //输出图片
    private function echoImg()
    {
        //文件类型验证
        if( !in_array($this->mimeType,['image/png','image/jpeg','image/jpeg','image/gif','image/webp','application/x-bmp','text/xml']) ){
            $this->redirect($this->imageUrl, 302);
        }
        //文件内容验证
        if(!$this->imgContent){
            $this->redirect($this->imageUrl, 302);
        }
        //ETAG浏览器缓存
        $iftag = isset($_SERVER['HTTP_IF_NONE_MATCH']) ? $_SERVER['HTTP_IF_NONE_MATCH'] == $this->fileName : null;
        if($iftag){
            header('HTTP/1.0 304 Not Modified');
            exit();
        }
        //header头
        header('Content-type: '.$this->mimeType);
        //浏览器缓存标识
        header('ETag: '.$this->fileName);
        //图片内容
        exit($this->imgContent);
    }
    
    //图片地址加密处理
    private function urlEncode($imgUrl){
        if(!$imgUrl){
            return $this->imageUrl;
        }
        if( config('tuchuang.link_encode') ){
            foreach( explode(',', config('tuchuang.link_encode')) as $key=>$function){
                $imgUrl = $function($imgUrl);
            }
        }
        return $imgUrl;
    }
    
    //图片地址解密处理
    private function urlDecode($imgUrl){
        if(!$imgUrl){
            return $this->imageUrl;
        }
        //$imgUrl = urldecode($imgUrl);
        if( config('tuchuang.link_decode') ){
            foreach( explode(',', config('tuchuang.link_decode')) as $key=>$function){
                $imgUrl = $function($imgUrl);
            }
        }
        return DcEmpty($imgUrl, $this->imageUrl);
    }
    
    //CURL获取文件内容
    private function curl($url='')
    {
        $ch = curl_init();
        curl_setopt ($ch, CURLOPT_URL, $url);
        curl_setopt ($ch, CURLOPT_HEADER, 0);
        curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1); 
        curl_setopt ($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT,10);
        curl_setopt ($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt ($ch, CURLOPT_ENCODING, "");
        //https自动处理
        $http = parse_url($url);
        if($http['scheme'] == 'https'){
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
        }
        //设置图片内容
        $this->imgContent = curl_exec($ch);
        //设置图片类型（CONTENT_TYPE）
        $contentType      = explode(";", curl_getinfo($ch, CURLINFO_CONTENT_TYPE));
        $this->mimeType   = $contentType[0];
        //关闭链接
        curl_close($ch);
    }
    
    public function _empty($name)
    {
        return DcHtml($name);
	}
}